searchNodes=[{"ref":"OffBroadway.Kafka.html","title":"OffBroadway.Kafka","type":"behaviour","doc":"Implements a macro for easily including the OffBroadway.Kafka library in your application and declaring the callbacks expected by the framework to configure it&#39;s Broadway and Kafka configurations. The OffBroadway.Kafka process creates an Elsa group supervisor as part of its setup to manage consumer group interaction with the cluster. OffBroadway.Kafka behaviour To define the custom Broadway behaviour for OffBroadway.Kafka: Define a broadway_config/3 function that receives a keyword list of general configuration values as well as a topic and a partition and constructs the necessary keyword list to configure the Broadway processers and any desired batchers and contexts. Define a kafka_config/1 function that constructs the necessary configuration to pass to Elsa for defining which Kafka brokers to connect to, which topic and partition Broadway should consume from, a name and consumer group name, as well as consumer configs: prefetch_count prefetch_bytes begin_offset Examples ClassicHandler This implementation uses Broadway directly and defines the handler to use the OffBroad.Kafka.Producer handler which communicates with the cluster via Elsa: defmodule ClassicBroadway do use Broadway def start_link(opts) do kafka_config = [ connection: :client1, endpoints: [localhost: 9092], group_consumer: [ group: &quot;classic&quot;, topics: [&quot;topic1&quot;], config: [ begin_offset: :earliest ] ] ] Broadway.start_link(__MODULE__, name: __MODULE__, producer: [ module: {OffBroadway.Kafka.Producer, kafka_config}, stages: 1 ], processors: [ default: [ stages: 1 ] ], context: %{pid: Keyword.get(opts, :pid)} ) end def handle_message(processor, message, context) do send(context.pid, {:message, message}) message end end ShowtimeHandler This implementation uses the OffBroadway.Kafka behaviour and starts an Elsa group supervisor directly to have the handler create a per-topic, per-partition basis Broadway pipeline on receiving partition assignments from the group coordinator for nested concurrency of processing events: defmodule ShowtimeBroadway do use OffBroadway.Kafka def kafka_config(_opts) do [ connection: :per_partition, endpoints: [localhost: 9092], group_consumer: [ group: &quot;per_partition&quot;, topics: [&quot;topic1&quot;], config: [ prefetch_count: 5, prefetch_bytes: 0, begin_offset: :earliest ] ] ] end def broadway_config(opts, topic, partition) do [ name: :&quot;broadway_per_partition_\#{topic}_\#{partition}&quot;, processors: [ default: [ stages: 5 ] ], context: %{ pid: Keyword.get(opts, :pid) } ] end def handle_message(processor, message, context) do send(context.pid, {:message, message}) message end end"},{"ref":"OffBroadway.Kafka.html#__using__/1","title":"OffBroadway.Kafka.__using__/1","type":"macro","doc":"Macro to include the setup functionality for OffBroadway.Kafka and start an Elsa consumer group manager for each Broadway pipeline instantiated."},{"ref":"OffBroadway.Kafka.html#c:broadway_config/3","title":"OffBroadway.Kafka.broadway_config/3","type":"callback","doc":""},{"ref":"OffBroadway.Kafka.html#c:kafka_config/1","title":"OffBroadway.Kafka.kafka_config/1","type":"callback","doc":""},{"ref":"OffBroadway.Kafka.Acknowledger.html","title":"OffBroadway.Kafka.Acknowledger","type":"module","doc":"Implements the Broadway acknowledger behaviour, handling acking of processed messages back to Kafka once they have been successfully processed. Message ack references are stored in ETS as an ordered set and acknowledgements are performed in the order received rather than the order processed to ensure that a failure of the Broadway pipeline does not allow messages received later but processed faster to erroneously mark lost messages as acknowledged when they should instead be reprocessed on recovery of the pipeline."},{"ref":"OffBroadway.Kafka.Acknowledger.html#ack/3","title":"OffBroadway.Kafka.Acknowledger.ack/3","type":"function","doc":"Acknowledges processed messages to Kafka. Due to Kafka&#39;s requirement to maintain message order for proper offset management, concatenates successful and failed messages together and stores the total offset to return for acknowledgement."},{"ref":"OffBroadway.Kafka.Acknowledger.html#ack_ref/1","title":"OffBroadway.Kafka.Acknowledger.ack_ref/1","type":"function","doc":"Constructs an ack_ref record for storing the status of message acknowledgement in ETS."},{"ref":"OffBroadway.Kafka.Acknowledger.html#add_offsets/2","title":"OffBroadway.Kafka.Acknowledger.add_offsets/2","type":"function","doc":"Adds a set of messages, represented by a contiguous range, to ETS for tracking acknowledgement in the proper order."},{"ref":"OffBroadway.Kafka.Acknowledger.html#child_spec/1","title":"OffBroadway.Kafka.Acknowledger.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"OffBroadway.Kafka.Acknowledger.html#is_empty?/1","title":"OffBroadway.Kafka.Acknowledger.is_empty?/1","type":"function","doc":""},{"ref":"OffBroadway.Kafka.Acknowledger.html#start_link/1","title":"OffBroadway.Kafka.Acknowledger.start_link/1","type":"function","doc":"Creates an acknowledger GenServer process and links it to the current process."},{"ref":"OffBroadway.Kafka.Acknowledger.html#t:ack_ref/0","title":"OffBroadway.Kafka.Acknowledger.ack_ref/0","type":"type","doc":""},{"ref":"OffBroadway.Kafka.ClassicHandler.html","title":"OffBroadway.Kafka.ClassicHandler","type":"module","doc":"Supplies a message handler for integrating with Broadway in a more traditional manner of including Broadway directly in the implementing application only supplying a handler module customized to the producing system. The &quot;Classic&quot; handler supplies a simple pass-through init function and delegates message handling to the OffBroadway.Kafka.Producer module."},{"ref":"OffBroadway.Kafka.ClassicHandler.html#connection/0","title":"OffBroadway.Kafka.ClassicHandler.connection/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ClassicHandler.html#generation_id/0","title":"OffBroadway.Kafka.ClassicHandler.generation_id/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ClassicHandler.html#handle_messages/1","title":"OffBroadway.Kafka.ClassicHandler.handle_messages/1","type":"function","doc":"Callback implementation for Elsa.Consumer.MessageHandler.handle_messages/1."},{"ref":"OffBroadway.Kafka.ClassicHandler.html#handle_messages/2","title":"OffBroadway.Kafka.ClassicHandler.handle_messages/2","type":"function","doc":"Delegates message processing to the Producer message handler function."},{"ref":"OffBroadway.Kafka.ClassicHandler.html#init/1","title":"OffBroadway.Kafka.ClassicHandler.init/1","type":"function","doc":"Supplies a basic init function, delegating the majority of setup to the included Elsa consumer handler using macro."},{"ref":"OffBroadway.Kafka.ClassicHandler.html#partition/0","title":"OffBroadway.Kafka.ClassicHandler.partition/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ClassicHandler.html#topic/0","title":"OffBroadway.Kafka.ClassicHandler.topic/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.Producer.html","title":"OffBroadway.Kafka.Producer","type":"module","doc":"Implements the logic to handle incoming messages through the broadway pipeline. Sends messages to the handle_info/2 and handle_demand/2 functions based on requests and tracks acknowledgements in state."},{"ref":"OffBroadway.Kafka.Producer.html#assignments_revoked/1","title":"OffBroadway.Kafka.Producer.assignments_revoked/1","type":"function","doc":""},{"ref":"OffBroadway.Kafka.Producer.html#handle_demand/2","title":"OffBroadway.Kafka.Producer.handle_demand/2","type":"function","doc":"Handle message events based on demand. Updates the demand based on the existing demand in state and sends the requested number of message events."},{"ref":"OffBroadway.Kafka.Producer.html#handle_info/2","title":"OffBroadway.Kafka.Producer.handle_info/2","type":"function","doc":"Handle assignments revoked by kafka broker. Should wait until acknowledgers and reported as empty and clean out any events in queue"},{"ref":"OffBroadway.Kafka.Producer.html#handle_messages/2","title":"OffBroadway.Kafka.Producer.handle_messages/2","type":"function","doc":"Convenience function for sending messages to be produced via the handle_info/2 function."},{"ref":"OffBroadway.Kafka.Producer.html#init/1","title":"OffBroadway.Kafka.Producer.init/1","type":"function","doc":"Names the producer process and initializes state for the producer GenServer. if args contain a value for :brokers or :endpoints, creates a handler config and passes it to the Elsa library to start a consumer group supervisor and store the returned pid in the Broadway Producer state for reference."},{"ref":"OffBroadway.Kafka.Producer.html#start_link/1","title":"OffBroadway.Kafka.Producer.start_link/1","type":"function","doc":"Starts an OffBroadway.Kafka producer process linked to the current process."},{"ref":"OffBroadway.Kafka.Producer.html#terminate/2","title":"OffBroadway.Kafka.Producer.terminate/2","type":"function","doc":"Handles termination of the Elsa consumer group supervisor process if one exists."},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html","title":"OffBroadway.Kafka.ShowtimeHandler","type":"module","doc":"Implements message handling using an opinionated interpretation of Broadway for interaction with Kafka. Intended for use in conjunction with the __using__ macro provided by OffBroadway.Kafka. Assumes a single producer stage, preconfigured to use the OffBroadway.Kafka.Producer module and takes configuration for additional Broadway elements via the Elsa configurations passed through via the kafka_config/1 function and those passed to Broadway via the broadway_config/1 functions defined in the top-level module&#39;s behaviour callbacks."},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#connection/0","title":"OffBroadway.Kafka.ShowtimeHandler.connection/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#generation_id/0","title":"OffBroadway.Kafka.ShowtimeHandler.generation_id/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#handle_messages/1","title":"OffBroadway.Kafka.ShowtimeHandler.handle_messages/1","type":"function","doc":"Callback implementation for Elsa.Consumer.MessageHandler.handle_messages/1."},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#handle_messages/2","title":"OffBroadway.Kafka.ShowtimeHandler.handle_messages/2","type":"function","doc":"Delegates messages to the processed to the Producer message handler function."},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#init/1","title":"OffBroadway.Kafka.ShowtimeHandler.init/1","type":"function","doc":"Assumes a single producer stage using the OffBroadway.Kafka.Producer module and collects other Broadway configuration passed in from the calling application&#39;s implementation of the broadway_config/1 behaviour. Retains a reference to the producer process in the state via Broadway&#39;s built-in Broadway.Server.get_random_producer/1 function to allow passing reference to the producer process to Elsa."},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#partition/0","title":"OffBroadway.Kafka.ShowtimeHandler.partition/0","type":"function","doc":""},{"ref":"OffBroadway.Kafka.ShowtimeHandler.html#topic/0","title":"OffBroadway.Kafka.ShowtimeHandler.topic/0","type":"function","doc":""}]